"""
Test models
"""

# Django
from django.contrib.auth.models import Group
from django.core.exceptions import ValidationError
from django.test import TestCase

# AA Fleet Pings
from fleetpings.models import FleetDoctrine, Webhook


class TestModels(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        """
        Set up groups and users
        """

        super().setUpClass()

        cls.group = Group.objects.create(name="Superhero")

    def test_discord_webhook_should_throw_exception(self):
        """
        Test if we get a ValidationError for a Discord webhook
        :return:
        """

        # given
        webhook = Webhook(
            url=(
                "https://discord.com/api/webhooks/754343402302911920F/x-BfFCdEG"
                "-qGg_39_mFwDU_vHdFUqRSLozaAxZdgKO2dm6Oa8vxNdAyesVpy-Bzf8pm-"
            ),
        )

        # when
        with self.assertRaises(ValidationError):
            webhook.clean()

        with self.assertRaisesMessage(
            ValidationError,
            expected_message=(
                "Invalid webhook URL. The webhook URL you entered does not match any "
                "known format for a Discord webhook. Please check the "
                "webhook URL."
            ),
        ):
            webhook.clean()

    def test_doctrine_link_should_throw_exception(self):
        """
        Test if we get a ValidationError for a doctrine link
        :return:
        """

        # given
        doctrine = FleetDoctrine(
            name="Awesome Doctrine",
            link=("htp://invalid-doctrine.url"),
        )

        # when
        with self.assertRaises(ValidationError):
            doctrine.clean()

        with self.assertRaisesMessage(
            ValidationError,
            expected_message=("Your doctrine URL is not valid."),
        ):
            doctrine.clean()

    # def test_discord_ping_target_should_throw_not_synced_yet_exception(self):
    #     """
    #     Test if we get a ValidationError for a doctrine link
    #     :return:
    #     """
    #
    #     # given
    #     ping_target = DiscordPingTargets(name=self.group)
    #
    #     # when
    #     with self.assertRaises(ValidationError):
    #         ping_target.clean()
    #
    #     with self.assertRaisesMessage(
    #         ValidationError,
    #         expected_message=("This group has not been synced to Discord yet."),
    #     ):
    #         ping_target.clean()



"""
Handling ping context data
"""

# Alliance Auth (External Libs)
from app_utils.urls import reverse_absolute

# AA Fleet Pings
from fleetpings.app_settings import (
    fittings_installed,
    optimer_installed,
    timezones_installed,
)
from fleetpings.constants import DEFAULT_EMBED_COLOR
from fleetpings.models import DiscordPingTargets, Webhook


def get_ping_context_from_form_data(form_data: dict) -> dict:
    """
    Getting ping context from form data
    :param form_data:
    :return:
    """

    ping_target_group_id = None
    ping_target_group_name = None
    ping_target_at_mention = None

    if form_data["ping_target"]:
        if (
            form_data["ping_target"] == "@here"
            or form_data["ping_target"] == "@everyone"
        ):
            ping_target_at_mention = str(form_data["ping_target"])
        else:
            try:
                # Check if we deal with a custom ping target
                ping_target = DiscordPingTargets.objects.get(
                    discord_id=form_data["ping_target"]
                )
            except DiscordPingTargets.DoesNotExist:
                pass
            else:
                # We deal with a custom ping target, gather the information we need
                ping_target_group_id = int(ping_target.discord_id)
                ping_target_group_name = str(ping_target.name)
                ping_target_at_mention = (
                    str(ping_target.name)
                    if str(ping_target.name).startswith("@")
                    else f"@{ping_target.name}"
                )

    # Check for webhooks
    ping_channel_webhook = None
    ping_channel_webhook_embed_color = DEFAULT_EMBED_COLOR

    if form_data["ping_channel"]:
        try:
            ping_channel = Webhook.objects.get(pk=form_data["ping_channel"])
        except Webhook.DoesNotExist:
            pass
        else:
            ping_channel_webhook = ping_channel.url

    if form_data["webhook_embed_color"]:
        ping_channel_webhook_embed_color = form_data["webhook_embed_color"]

    ping_context = {
        "ping_target": {
            "group_id": int(ping_target_group_id) if ping_target_group_id else None,
            "group_name": str(ping_target_group_name),
            "at_mention": str(ping_target_at_mention) if ping_target_at_mention else "",
        },
        "ping_channel": {
            "webhook": ping_channel_webhook,
            "embed_color": ping_channel_webhook_embed_color,
        },
        "fleet_type": str(form_data["fleet_type"]),
        "fleet_commander": str(form_data["fleet_commander"]),
        "fleet_name": str(form_data["fleet_name"]),
        "formup_location": str(form_data["formup_location"]),
        "is_pre_ping": bool(form_data["pre_ping"]),
        "is_formup_now": bool(form_data["formup_now"]),
        "formup_time": {
            "datetime_string": str(form_data["formup_time"]),
            "timestamp": str(form_data["formup_timestamp"]),
        },
        "fleet_comms": str(form_data["fleet_comms"]),
        "doctrine": {
            "name": str(form_data["fleet_doctrine"]),
            "link": str(form_data["fleet_doctrine_url"]),
        },
        "srp": {
            "has_srp": bool(form_data["srp"]),
            "create_srp_link": bool(form_data["srp_link"]),
        },
        "create_optimer": bool(form_data["optimer"]),
        "additional_information": str(form_data["additional_information"]),
        "timezones_installed": timezones_installed(),
        "optimer_installed": optimer_installed(),
        "fittings_installed": fittings_installed(),
    }

    return ping_context


def _get_webhook_ping_context(ping_context: dict) -> dict:
    """
    Getting the webhook ping context
    :param ping_context:
    :return:
    """

    webhook_ping_text_header = ""
    webhook_ping_text_content = ""
    webhook_ping_text_footer = ""
    webhook_ping_target = ""

    # Ping target
    if ping_context["ping_target"]["group_id"]:
        ping_target_at_mention = f'<@&{ping_context["ping_target"]["group_id"]}>'
    else:
        ping_target_at_mention = str(ping_context["ping_target"]["at_mention"])

    webhook_ping_text_header += ping_target_at_mention
    webhook_ping_text_header += " :: "

    webhook_ping_text_header += "**"

    # Check if it's a pre-ping or not
    if ping_context["is_pre_ping"]:
        webhook_ping_text_header += "### PRE PING ###"

        if ping_context["fleet_type"]:
            webhook_ping_text_header += (
                f' / (Upcoming) {ping_context["fleet_type"]} Fleet'
            )
    else:
        if ping_context["fleet_type"]:
            webhook_ping_text_header += f'{ping_context["fleet_type"]} '

        webhook_ping_text_header += "Fleet is up"

        # Add fcName if we have one
        if ping_context["fleet_commander"]:
            webhook_ping_text_header += f' under {ping_context["fleet_commander"]}'

    webhook_ping_text_header += "**\n** **"

    # Add FC name if we have one
    if ping_context["fleet_commander"]:
        webhook_ping_text_content += f'\n**FC:** {ping_context["fleet_commander"]}'

    # Check if fleet name is available
    if ping_context["fleet_name"]:
        webhook_ping_text_content += f'\n**Fleet Name:** {ping_context["fleet_name"]}'

    # Check if form-up location is available
    if ping_context["formup_location"]:
        webhook_ping_text_content += (
            f'\n**Formup Location:** {ping_context["formup_location"]}'
        )

    # Check if form-up time is available
    if ping_context["is_formup_now"]:
        webhook_ping_text_content += "\n**Formup Time:** NOW"
    else:
        if ping_context["formup_time"]["datetime_string"]:
            webhook_ping_text_content += f'\n**Formup (EVE Time):** {ping_context["formup_time"]["datetime_string"]}'

        if ping_context["formup_time"]["timestamp"]:
            if timezones_installed():
                # Add timezones conversion to ping text
                timezones_url = reverse_absolute(
                    "timezones:index",
                    args=[ping_context["formup_time"]["timestamp"]],
                )
                webhook_ping_text_content += (
                    f" ([Time Zone Conversion]({timezones_url}))"
                )

            # Add local time
            webhook_ping_text_content += (
                "\n**Formup (Local Time):** "
                f'<t:{ping_context["formup_time"]["timestamp"]}:F>'
            )
    # Check if fleet comms is available
    if ping_context["fleet_comms"]:
        webhook_ping_text_content += f'\n**Comms:** {ping_context["fleet_comms"]}'

    # Check if doctrine is available
    if ping_context["doctrine"]["name"]:
        webhook_ping_text_content += (
            f'\n**Ships / Doctrine:** {ping_context["doctrine"]["name"]}'
        )

        # Grab the doctrine link if there is one
        if ping_context["doctrine"]["link"]:
            webhook_ping_text_content += (
                f' ([Doctrine Link]({ping_context["doctrine"]["link"]}))'
            )

    # Check if srp is available
    if ping_context["srp"]["has_srp"]:
        webhook_ping_text_content += "\n**SRP:** Yes"

        # Check if we have an SRP link
        if ping_context["srp"]["create_srp_link"] and "link" in ping_context["srp"]:
            webhook_ping_text_content += (
                f' (SRP Code: [{ping_context["srp"]["link"]["code"]}]'
                f'({ping_context["srp"]["link"]["link"]}))'
            )

    # Check if additional information is available
    if ping_context["additional_information"]:
        webhook_ping_text_content += (
            f'\n\n**Additional Information**:\n{ping_context["additional_information"]}'
        )

    return {
        "target": webhook_ping_target,
        "header": webhook_ping_text_header,
        "content": webhook_ping_text_content,
        "footer": webhook_ping_text_footer,
    }



# Generated by Django 2.2.16 on 2020-09-14 16:55

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("fleetpings", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="webhook",
            name="is_embedded",
            field=models.BooleanField(
                db_index=True,
                default=True,
                help_text=(
                    "Whether this webhook's ping is embedded or not. "
                    "(This setting only effects Discord webhooks.)"
                ),
            ),
        ),
    ]



"""
Our models
"""

# Standard Library
import re

# Third Party
from requests.exceptions import HTTPError

# Django
from django.contrib.auth.models import Group
from django.core.exceptions import ValidationError
from django.core.validators import URLValidator
from django.db import models
from django.utils.translation import gettext_lazy as _

# AA Fleet Pings
from fleetpings.app_settings import discord_service_installed

# Check if the Discord service is active
from fleetpings.constants import DISCORD_WEBHOOK_REGEX

if discord_service_installed():
    # Alliance Auth
    from allianceauth.services.modules.discord.models import DiscordUser


class AaFleetpings(models.Model):
    """
    Meta model for app permissions
    """

    class Meta:  # pylint: disable=too-few-public-methods
        """
        AaFleetpings :: Meta
        """

        managed = False
        default_permissions = ()
        permissions = (("basic_access", "Can access this app"),)


# FleetComm Model
class FleetComm(models.Model):
    """
    Fleet Comms
    """

    name = models.CharField(
        max_length=255, unique=True, help_text=_("Short name to identify this comms")
    )

    notes = models.TextField(
        null=True,
        blank=True,
        help_text=_("You can add notes about this configuration here if you want"),
    )

    is_enabled = models.BooleanField(
        default=True,
        db_index=True,
        help_text=_("Whether this comms is enabled or not"),
    )

    def __str__(self) -> str:
        return str(self.name)

    class Meta:  # pylint: disable=too-few-public-methods
        """
        FleetComm :: Meta
        """

        verbose_name = _("Fleet Comm")
        verbose_name_plural = _("Fleet Comms")
        default_permissions = ()


# FleetDoctrine Model
class FleetDoctrine(models.Model):
    """
    Fleet Doctrine
    """

    # Doctrine name
    name = models.CharField(
        max_length=255, unique=True, help_text=_("Short name to identify this doctrine")
    )

    # Link to your doctrine
    link = models.CharField(
        max_length=255,
        help_text=_("A link to a doctrine page for this doctrine if you have."),
        blank=True,
    )

    # Restrictions
    restricted_to_group = models.ManyToManyField(
        Group,
        blank=True,
        related_name="fleetdoctrine_require_groups",
        help_text=_("Restrict this doctrine to the following group(s) ..."),
    )

    # Doctrine notes
    notes = models.TextField(
        null=True,
        blank=True,
        help_text=_("You can add notes about this configuration here if you want"),
    )

    # Is doctrine active
    is_enabled = models.BooleanField(
        default=True,
        db_index=True,
        help_text=_("Whether this doctrine is enabled or not"),
    )

    def clean(self):
        """
        Check if the doctrine link is an actual link to a website
        """

        doctrine_link = self.link

        if doctrine_link != "":
            validate = URLValidator()

            try:
                validate(doctrine_link)
            except ValidationError as exception:
                raise ValidationError(
                    _("Your doctrine URL is not valid.")
                ) from exception

        super().clean()

    def __str__(self) -> str:
        return str(self.name)

    class Meta:  # pylint: disable=too-few-public-methods
        """
        FleetDoctrine :: Meta
        """

        verbose_name = _("Fleet Doctrine")
        verbose_name_plural = _("Fleet Doctrines")
        default_permissions = ()


# FormupLocation Model
class FormupLocation(models.Model):
    """
    Formup Location
    """

    # formup location name
    name = models.CharField(
        max_length=255,
        unique=True,
        help_text=_("Short name to identify this formup location"),
    )

    # Formup location notes
    notes = models.TextField(
        null=True,
        blank=True,
        help_text=_("You can add notes about this configuration here if you want"),
    )

    # Is formup location active
    is_enabled = models.BooleanField(
        default=True,
        db_index=True,
        help_text=_("Whether this formup location is enabled or not"),
    )

    def __str__(self) -> str:
        return str(self.name)

    class Meta:  # pylint: disable=too-few-public-methods
        """
        FormupLocation :: Meta
        """

        verbose_name = _("Formup Location")
        verbose_name_plural = _("Formup Locations")
        default_permissions = ()


# DiscordPingTargets Model
class DiscordPingTargets(models.Model):
    """
    Discord Ping Targets
    """

    # Discord group to ping
    name = models.OneToOneField(
        Group,
        on_delete=models.CASCADE,
        unique=True,
        help_text=(
            _(
                "Name of the Discord role to ping. "
                "(Note: This must be an Auth group that is synched to Discord.)"
            )
        ),
    )

    # Discord group id
    discord_id = models.CharField(
        max_length=255,
        unique=True,
        blank=True,
        help_text=_("ID of the Discord role to ping"),
    )

    # Restrictions
    restricted_to_group = models.ManyToManyField(
        Group,
        blank=True,
        related_name="discord_role_require_groups",
        help_text=_("Restrict ping rights to the following group(s) ..."),
    )

    # Notes
    notes = models.TextField(
        null=True,
        blank=True,
        help_text=_("You can add notes about this configuration here if you want"),
    )

    # Is this group active
    is_enabled = models.BooleanField(
        default=True,
        db_index=True,
        help_text=_("Whether this formup location is enabled or not"),
    )

    def clean(self):
        """
        Check if the group has already been synced to Discord,
        if not, raise an error
        """

        # Check if the Discord service is active
        if not discord_service_installed():
            raise ValidationError(
                _("You might want to install the Discord service first ...")
            )

        # Get the group id from Discord
        try:
            discord_group_info = DiscordUser.objects.group_to_role(self.name)
        except HTTPError as http_error:
            raise ValidationError(
                _("Are you sure you have your Discord linked to your Alliance Auth?")
            ) from http_error
        else:
            if not discord_group_info:
                raise ValidationError(
                    _("This group has not been synced to Discord yet.")
                )

        super().clean()

    def save(
        self, force_insert=False, force_update=False, using=None, update_fields=None
    ):
        """
        Add the Discord group ID (if Discord service is active) and save the whole thing
        """

        # Check if the Discord service is active
        if discord_service_installed():
            discord_group_info = DiscordUser.objects.group_to_role(self.name)
            self.discord_id = discord_group_info["id"]

        super().save()  # Call the "real" save() method.

    def __str__(self) -> str:
        return str(self.name)

    class Meta:  # pylint: disable=too-few-public-methods
        """
        DiscordPingTargets :: Meta
        """

        verbose_name = _("Discord Ping Target")
        verbose_name_plural = _("Discord Ping Targets")
        default_permissions = ()


# FleetType Model
class FleetType(models.Model):
    """
    Fleet Types
    """

    # Name of the fleet type
    name = models.CharField(
        max_length=255,
        unique=True,
        help_text=_("Short name to identify this fleet type"),
    )

    # Embed color
    embed_color = models.CharField(
        max_length=7,
        blank=True,
        help_text=_("Hightlight color for the embed"),
    )

    # Restrictions
    restricted_to_group = models.ManyToManyField(
        Group,
        blank=True,
        related_name="fleettype_require_groups",
        help_text=_("Restrict this fleet type to the following group(s) ..."),
    )

    # Fleet type notes
    notes = models.TextField(
        null=True,
        blank=True,
        help_text=_("You can add notes about this configuration here if you want"),
    )

    # Is this fleet type enabled
    is_enabled = models.BooleanField(
        default=True,
        db_index=True,
        help_text=_("Whether this fleet type is enabled or not"),
    )

    def __str__(self) -> str:
        return str(self.name)

    class Meta:  # pylint: disable=too-few-public-methods
        """
        FleetType :: Meta
        """

        verbose_name = _("Fleet Type")
        verbose_name_plural = _("Fleet Types")
        default_permissions = ()


# Webhook Model
class Webhook(models.Model):
    """
    A Discord webhook
    """

    # Channel name
    name = models.CharField(
        max_length=255,
        unique=True,
        help_text=_("Name of the channel this webhook posts to"),
    )

    # Wehbook url
    url = models.CharField(
        max_length=255,
        unique=True,
        help_text=(
            _(
                "URL of this webhook, e.g. "
                "https://discord.com/api/webhooks/123456/abcdef"
            )
        ),
    )

    # Restrictions
    restricted_to_group = models.ManyToManyField(
        Group,
        blank=True,
        related_name="webhook_require_groups",
        help_text=_("Restrict ping rights to the following group(s) ..."),
    )

    # Webhook notes
    notes = models.TextField(
        null=True,
        blank=True,
        help_text=_("You can add notes about this webhook here if you want"),
    )

    # Is it enabled
    is_enabled = models.BooleanField(
        default=True,
        db_index=True,
        help_text=_("Whether this webhook is active or not"),
    )

    def __str__(self) -> str:
        return str(self.name)

    class Meta:  # pylint: disable=too-few-public-methods
        """
        Webhook :: Meta
        """

        verbose_name = _("Webhook")
        verbose_name_plural = _("Webhooks")
        default_permissions = ()

    def clean(self):
        """
        Check if the webhook URL is valid
        :return:
        """

        # Check if it's an actual kill mail
        if not re.match(DISCORD_WEBHOOK_REGEX, self.url):
            raise ValidationError(
                _(
                    "Invalid webhook URL. The webhook URL you entered does not match "
                    "any known format for a Discord webhook. Please check the "
                    "webhook URL."
                )
            )

        super().clean()



"""
Test checks for installed modules we might use
"""

# Django
from django.test import TestCase, modify_settings

# AA Fleet Pings
from fleetpings.app_settings import (
    discord_service_installed,
    optimer_installed,
    srp_module_installed,
    srp_module_is,
    timezones_installed,
)


class TestModulesInstalled(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        """
        Set up groups and users
        """

        super().setUpClass()

    @modify_settings(INSTALLED_APPS={"remove": "timezones"})
    def test_for_timezones_installed_when_not_installed(self):
        """
        Test for timezones_installed when it is not
        :return:
        """

        self.assertFalse(timezones_installed())

    @modify_settings(INSTALLED_APPS={"append": "timezones"})
    def test_for_timezones_installed_when_installed(self):
        """
        Test for timezones_installed when it is installed
        :return:$FilePath$
        """

        self.assertTrue(timezones_installed())

    @modify_settings(INSTALLED_APPS={"remove": "allianceauth.optimer"})
    def test_for_optimer_installed_when_not_installed(self):
        """
        Test for optimer_installed when it is not
        :return:
        """

        self.assertFalse(optimer_installed())

    @modify_settings(INSTALLED_APPS={"append": "allianceauth.optimer"})
    def test_for_optimer_installed_when_installed(self):
        """
        Test for optimer_installed when it is installed
        :return:
        """

        self.assertTrue(optimer_installed())

    @modify_settings(INSTALLED_APPS={"remove": "allianceauth.services.modules.discord"})
    def test_for_discord_service_installed_when_not_installed(self):
        """
        Test for discord_service_installed when it is not
        :return:
        """

        self.assertFalse(discord_service_installed())

    @modify_settings(INSTALLED_APPS={"append": "allianceauth.services.modules.discord"})
    def test_for_discord_service_installed_when_installed(self):
        """
        Test for discord_service_installed when it is installed
        :return:
        """

        self.assertTrue(discord_service_installed())

    @modify_settings(INSTALLED_APPS={"remove": ["allianceauth.srp", "aasrp"]})
    def test_for_srp_module_installed_when_not_installed(self):
        """
        Test for srp_module_installed when it is not
        :return:
        """

        self.assertFalse(srp_module_installed())

    @modify_settings(
        INSTALLED_APPS={
            "remove": ["allianceauth.srp", "aasrp"],  # Remove all SRP modules
            "append": "aasrp",  # Add aasrp
        }
    )
    def test_for_srp_module_installed_when_aasrp_installed(self):
        """
        Test for srp_module_installed when it is aasrp
        :return:
        """

        self.assertTrue(srp_module_installed())
        self.assertTrue(srp_module_is("aasrp"))

    @modify_settings(
        INSTALLED_APPS={
            "remove": ["allianceauth.srp", "aasrp"],  # Remove all SRP modules
            "append": "allianceauth.srp",  # Add allianceauth.srp
        }
    )
    def test_for_srp_module_installed_when_allianceauth_srp_installed(self):
        """
        Test for srp_module_installed when it is allianceauth.srp
        :return:
        """

        self.assertTrue(srp_module_installed())
        self.assertTrue(srp_module_is("allianceauth.srp"))



"""
Test checks for access to fleetpings
"""

# Django
from django.contrib.auth.models import Group
from django.test import TestCase, modify_settings
from django.urls import reverse

# AA Fleet Pings
from fleetpings.tests.utils import create_fake_user


class TestAccess(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        """
        Set up groups and users
        """

        super().setUpClass()

        cls.group = Group.objects.create(name="Superhero")

        # User cannot access fleetpings
        cls.user_1001 = create_fake_user(1001, "Peter Parker")

        # User can access fleetpings
        cls.user_1002 = create_fake_user(
            1002, "Bruce Wayne", permissions=["fleetpings.basic_access"]
        )

        # User can add srp (aasrp)
        cls.user_1003 = create_fake_user(
            1003,
            "Clark Kent",
            permissions=["fleetpings.basic_access", "aasrp.create_srp"],
        )

    def test_has_no_access(self):
        """
        Test that a user without access get a 302
        :return:
        """

        # given
        self.client.force_login(self.user_1001)

        # when
        res = self.client.get(reverse("fleetpings:index"))

        # then
        self.assertEqual(res.status_code, 302)

    def test_has_access(self):
        """
        Test that a user with access get to see it
        :return:
        """

        # given
        self.client.force_login(self.user_1002)

        # when
        res = self.client.get(reverse("fleetpings:index"))

        # then
        self.assertEqual(res.status_code, 200)

    @modify_settings(
        INSTALLED_APPS={
            "remove": [
                "allianceauth.srp",
                "aasrp",
                "timezones",
                "allianceauth.optimer",
                "fittings",
            ]
        }
    )
    def test_has_access_without_additional_modules(self):
        """
        Test that a user with access get to see it
        :return:
        """

        # given
        self.client.force_login(self.user_1002)

        # when
        response = self.client.get(reverse("fleetpings:index"))

        # then
        self.assertEqual(response.status_code, 200)

        # context data
        self.assertEqual(response.context["optimer_installed"], False)
        self.assertEqual(response.context["fittings_installed"], False)
        self.assertEqual(response.context["srp_module_available_to_user"], False)

    @modify_settings(
        INSTALLED_APPS={
            "remove": [
                "allianceauth.srp",
                "aasrp",
                "timezones",
                "allianceauth.optimer",
                "fittings",
            ],
            "append": "aasrp",
        }
    )
    def test_has_access_with_aasrp(self):
        """
        Test that a user with access get to see it
        :return:
        """

        # given
        self.client.force_login(self.user_1003)

        # when
        response = self.client.get(reverse("fleetpings:index"))

        # then
        self.assertEqual(response.status_code, 200)

        # context data
        self.assertEqual(response.context["optimer_installed"], False)
        self.assertEqual(response.context["fittings_installed"], False)
        self.assertEqual(response.context["srp_module_available_to_user"], True)

    @modify_settings(
        INSTALLED_APPS={
            "remove": [
                "allianceauth.srp",
                "aasrp",
                "timezones",
                "allianceauth.optimer",
                "fittings",
            ],
            "append": "allianceauth.srp",
        }
    )
    def test_has_access_with_allianceauth_srp(self):
        """
        Test that a user with access get to see it
        :return:
        """

        # given
        self.client.force_login(self.user_1003)

        # when
        response = self.client.get(reverse("fleetpings:index"))

        # then
        self.assertEqual(response.status_code, 200)

        # context data
        self.assertEqual(response.context["optimer_installed"], False)
        self.assertEqual(response.context["fittings_installed"], False)
        self.assertEqual(response.context["srp_module_available_to_user"], True)

    @modify_settings(
        INSTALLED_APPS={
            "remove": [
                "allianceauth.srp",
                "aasrp",
                "timezones",
                "allianceauth.optimer",
                "fittings",
            ],
            "append": "timezones",
        }
    )
    def test_has_access_with_timezones(self):
        """
        Test that a user with access get to see it
        :return:
        """

        # given
        self.client.force_login(self.user_1003)

        # when
        response = self.client.get(reverse("fleetpings:index"))

        # then
        self.assertEqual(response.status_code, 200)

        # context data
        self.assertEqual(response.context["optimer_installed"], False)
        self.assertEqual(response.context["fittings_installed"], False)
        self.assertEqual(response.context["srp_module_available_to_user"], False)

    @modify_settings(
        INSTALLED_APPS={
            "remove": [
                "allianceauth.srp",
                "aasrp",
                "timezones",
                "allianceauth.optimer",
                "fittings",
            ],
            "append": "allianceauth.optimer",
        }
    )
    def test_has_access_with_allianceauth_optimer(self):
        """
        Test that a user with access get to see it
        :return:
        """

        # given
        self.client.force_login(self.user_1003)

        # when
        response = self.client.get(reverse("fleetpings:index"))

        # then
        self.assertEqual(response.status_code, 200)

        # context data
        self.assertEqual(response.context["optimer_installed"], True)
        self.assertEqual(response.context["fittings_installed"], False)
        self.assertEqual(response.context["srp_module_available_to_user"], False)



# Generated by Django 2.2.16 on 2020-09-16 17:56

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("auth", "0011_update_proxy_permissions"),
        ("fleetpings", "0004_auto_20200915_1617"),
    ]

    operations = [
        migrations.AddField(
            model_name="fleettype",
            name="restricted_to_group",
            field=models.ManyToManyField(
                blank=True,
                help_text="Restrict this fleet type to the following group(s) ...",
                related_name="fleettype_require_groups",
                to="auth.Group",
            ),
        ),
    ]



"""
Versioned static URLs to break browser caches when changing the app version
"""

# Django
from django.template.defaulttags import register
from django.templatetags.static import static

# AA Fleet Pings
from fleetpings import __version__


@register.simple_tag
def fleetpings_static(path: str) -> str:
    """
    Versioned static URL
    :param path:
    :type path:
    :return:
    :rtype:
    """

    static_url = static(path)
    versioned_url = static_url + "?v=" + __version__

    return versioned_url



"""
Pages url config
"""

# Django
from django.urls import path

# AA Fleet Pings
from fleetpings import views

app_name: str = "fleetpings"

urlpatterns = [
    path("", views.index, name="index"),
    # Ajax calls
    path(
        "ajax/create-fleet-ping/",
        views.ajax_create_fleet_ping,
        name="ajax_create_fleet_ping",
    ),
    path(
        "ajax/get-ping-targets-for-user/",
        views.ajax_get_ping_targets,
        name="ajax_get_ping_targets",
    ),
    path(
        "ajax/get-webhooks-for-user/",
        views.ajax_get_webhooks,
        name="ajax_get_webhooks",
    ),
    path(
        "ajax/get-fleet-types-for-user/",
        views.ajax_get_fleet_types,
        name="ajax_get_fleet_types",
    ),
    path(
        "ajax/get-formup-locations/",
        views.ajax_get_formup_locations,
        name="ajax_get_formup_locations",
    ),
    path(
        "ajax/get-fleet-comms/",
        views.ajax_get_fleet_comms,
        name="ajax_get_fleet_comms",
    ),
    path(
        "ajax/get-fleet-doctrines-for-user/",
        views.ajax_get_fleet_doctrines,
        name="ajax_get_fleet_doctrines",
    ),
]



# Generated by Django 4.0.3 on 2022-04-16 22:23

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("fleetpings", "0008_auto_20210114_1733"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="webhook",
            name="is_embedded",
        ),
        migrations.RemoveField(
            model_name="webhook",
            name="type",
        ),
        migrations.AlterField(
            model_name="webhook",
            name="url",
            field=models.CharField(
                help_text="URL of this webhook, e.g. https://discord.com/api/webhooks/123456/abcdef",
                max_length=255,
                unique=True,
            ),
        ),
    ]



# Generated by Django 2.2.16 on 2020-09-14 19:44

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("fleetpings", "0002_webhook_is_embed_description_change"),
    ]

    operations = [
        migrations.AddField(
            model_name="fleetdoctrine",
            name="link",
            field=models.CharField(
                blank=True,
                help_text="A link to a doctrine page for this doctrine if you have.",
                max_length=255,
            ),
        ),
    ]



"""
Form declarations
"""

# Django
from django import forms
from django.utils.translation import gettext_lazy as _

# AA Fleet Pings
from fleetpings.app_settings import timezones_installed
from fleetpings.models import FleetType


def _get_timezones_module_hint_text() -> str:
    if timezones_installed():
        return _(
            " Timezones module is installed. Link to time zone conversion will be "
            "added automatically if you set a date and time here."
        )

    return ""


class FleetTypeAdminForm(forms.ModelForm):
    """
    Form definitions for the FleetType form in admin
    """

    class Meta:  # pylint: disable=too-few-public-methods
        """
        Meta
        """

        model = FleetType
        fields = "__all__"
        widgets = {"embed_color": forms.TextInput(attrs={"type": "color"})}


class FleetPingForm(forms.Form):
    """
    Fleet Ping Form
    """

    ping_target = forms.CharField(
        required=False,
        label=_("Ping Target"),
        widget=forms.Select(choices={}),
        help_text=_("Who do you want to ping?"),
    )
    pre_ping = forms.BooleanField(
        initial=False,
        required=False,
        label=_("Pre-Ping"),
        help_text=_("Mark this checkbox if this should be a pre-ping."),
    )
    ping_channel = forms.CharField(
        required=False,
        label=_("Ping To"),
        widget=forms.Select(choices={}),
        help_text=_("Select a channel to ping automatically"),
    )
    fleet_type = forms.CharField(
        required=False, label=_("Fleet Type"), widget=forms.Select(choices={})
    )
    fleet_commander = forms.CharField(
        required=False,
        label=_("FC Name"),
        max_length=254,
        widget=forms.TextInput(attrs={"placeholder": _("Who is the FC?")}),
    )
    fleet_name = forms.CharField(
        required=False,
        label=_("Fleet Name"),
        max_length=254,
        widget=forms.TextInput(
            attrs={"placeholder": _("What is the fleet name in fleet finder?")}
        ),
    )
    formup_location = forms.CharField(
        required=False,
        label=_("Formup Location"),
        widget=forms.TextInput(attrs={"list": "formupLocationList"}),
    )
    formup_time = forms.CharField(
        required=False,
        label=_("Formup Time"),
        max_length=254,
        widget=forms.TextInput(attrs={"disabled": "disabled", "autocomplete": "off"}),
        help_text=_(
            "To enable this field, either make it a Pre-Ping (checkbox above) or "
            "uncheck &quot;Formup NOW&quot; (checkbox below)."
        )
        + _get_timezones_module_hint_text(),
    )
    formup_timestamp = forms.CharField(
        required=False,
        label=_("Formup Timestamp"),
        widget=forms.TextInput(attrs={"hidden": "hidden"}),
    )
    formup_now = forms.BooleanField(
        initial=True,
        required=False,
        label=_("Formup NOW"),
        help_text=_(
            "If this checkbox is active, formup time will be set to &quot;NOW&quot; "
            "and the time in the field above (if any is set) will be ignored."
        ),
    )
    fleet_comms = forms.CharField(
        required=False,
        label=_("Fleet Comms"),
        widget=forms.TextInput(attrs={"list": "fleetCommsList"}),
    )
    fleet_doctrine = forms.CharField(
        required=False,
        label=_("Doctrine"),
        widget=forms.TextInput(attrs={"list": "fleetDoctrineList"}),
    )
    fleet_doctrine_url = forms.CharField(
        required=False,
        label=_("Doctrine URL"),
        widget=forms.TextInput(attrs={"hidden": "hidden"}),
    )
    webhook_embed_color = forms.CharField(
        required=False,
        label=_("Webhook Embed Color"),
        widget=forms.TextInput(attrs={"hidden": "hidden"}),
    )
    srp = forms.BooleanField(
        initial=False,
        required=False,
        label=_("SRP"),
        help_text=_("Is this fleet covered by SRP?"),
    )
    srp_link = forms.BooleanField(
        initial=False,
        required=False,
        label=_("Create SRP Link"),
        help_text=_(
            "If this checkbox is active, a SRP link specific for this fleet will be "
            "created.<br>Leave blank if unsure."
        ),
    )
    additional_information = forms.CharField(
        required=False,
        label=_("Additional Information"),
        widget=forms.Textarea(
            attrs={
                "rows": 10,
                "cols": 20,
                "input_type": "textarea",
                "placeholder": _(
                    "Feel free to add some more information about the fleet ..."
                ),
            }
        ),
    )
    optimer = forms.BooleanField(
        initial=False,
        required=False,
        label=_("Create Optimer"),
        help_text=_(
            "If this checkbox is active, a fleet operations timer for this pre-ping "
            "will be created."
        ),
    )



"""
The views
"""

# Standard Library
import json

# Django
from django.contrib.auth.decorators import login_required, permission_required
from django.core.handlers.wsgi import WSGIRequest
from django.db.models import Q
from django.http import HttpResponse
from django.shortcuts import render
from django.template.loader import render_to_string
from django.utils import timezone
from django.utils.translation import gettext_lazy as _

# Alliance Auth
from allianceauth.services.hooks import get_extension_logger

# Alliance Auth (External Libs)
from app_utils.logging import LoggerAddTag
from app_utils.urls import reverse_absolute, site_absolute_url

# AA Fleet Pings
from fleetpings import __title__
from fleetpings.app_settings import (
    AA_FLEETPINGS_USE_DOCTRINES_FROM_FITTINGS_MODULE,
    can_add_srp_links,
    fittings_installed,
    optimer_installed,
    srp_module_installed,
    srp_module_is,
)
from fleetpings.form import FleetPingForm
from fleetpings.helper.discord_webhook import ping_discord_webhook
from fleetpings.helper.ping_context import get_ping_context_from_form_data
from fleetpings.models import (
    DiscordPingTargets,
    FleetComm,
    FleetDoctrine,
    FleetType,
    FormupLocation,
    Webhook,
)

if (
    fittings_installed() is True
    and AA_FLEETPINGS_USE_DOCTRINES_FROM_FITTINGS_MODULE is True
):
    # Third Party
    from fittings.views import _get_docs_qs


logger = LoggerAddTag(get_extension_logger(__name__), __title__)


@login_required
@permission_required("fleetpings.basic_access")
def index(request: WSGIRequest) -> HttpResponse:
    """
    Index view
    """

    logger.info(f"Fleet pings view called by user {request.user}")

    srp_module_available_to_user = False
    if srp_module_installed() and (
        can_add_srp_links(request=request, module_name="aasrp")
        or can_add_srp_links(request=request, module_name="allianceauth.srp")
    ):
        srp_module_available_to_user = True

    context = {
        "title": __title__,
        "webhooks_configured": Webhook.objects.filter(
            Q(restricted_to_group__in=request.user.groups.all())
            | Q(restricted_to_group__isnull=True),
            is_enabled=True,
        ).exists(),
        "site_url": site_absolute_url(),
        # "timezones_installed": timezones_installed(),
        "optimer_installed": optimer_installed(),
        "fittings_installed": fittings_installed(),
        "main_character": request.user.profile.main_character,
        "srp_module_available_to_user": srp_module_available_to_user,
        "form": FleetPingForm,
    }

    return render(request, "fleetpings/index.html", context)


@login_required
@permission_required("fleetpings.basic_access")
def ajax_get_ping_targets(request: WSGIRequest) -> HttpResponse:
    """
    Get ping targets for the current user
    :param request:
    :return:
    """

    logger.info(f"Getting ping targets for user {request.user}")

    additional_discord_ping_targets = (
        DiscordPingTargets.objects.filter(
            Q(restricted_to_group__in=request.user.groups.all())
            | Q(restricted_to_group__isnull=True),
            is_enabled=True,
        )
        .distinct()
        .order_by("name")
    )

    return render(
        request,
        "fleetpings/partials/form/segments/ping-targets.html",
        {"ping_targets": additional_discord_ping_targets},
    )


@login_required
@permission_required("fleetpings.basic_access")
def ajax_get_webhooks(request: WSGIRequest) -> HttpResponse:
    """
    Get webhooks for ccurrent user
    :param request:
    :return:
    """

    logger.info(f"Getting webhooks for user {request.user}")

    webhooks = (
        Webhook.objects.filter(
            Q(restricted_to_group__in=request.user.groups.all())
            | Q(restricted_to_group__isnull=True),
            is_enabled=True,
        ).distinct()
        # .order_by("type", "name")
    )

    return render(
        request,
        "fleetpings/partials/form/segments/ping-channel.html",
        {"webhooks": webhooks},
    )


@login_required
@permission_required("fleetpings.basic_access")
def ajax_get_fleet_types(request: WSGIRequest) -> HttpResponse:
    """
    Get fleet types for current user
    :param request:
    :return:
    """

    logger.info(f"Getting fleet types for user {request.user}")

    fleet_types = (
        FleetType.objects.filter(
            Q(restricted_to_group__in=request.user.groups.all())
            | Q(restricted_to_group__isnull=True),
            is_enabled=True,
        )
        .distinct()
        .order_by("name")
    )

    return render(
        request,
        "fleetpings/partials/form/segments/fleet-type.html",
        {"fleet_types": fleet_types},
    )


@login_required
@permission_required("fleetpings.basic_access")
def ajax_get_formup_locations(request: WSGIRequest) -> HttpResponse:
    """
    Get formup locations
    :param request:
    :return:
    """

    logger.info(f"Getting formup locations for user {request.user}")

    formup_locations = FormupLocation.objects.filter(is_enabled=True).order_by("name")

    return render(
        request,
        "fleetpings/partials/form/segments/fleet-formup-location.html",
        {"formup_locations": formup_locations},
    )


@login_required
@permission_required("fleetpings.basic_access")
def ajax_get_fleet_comms(request: WSGIRequest) -> HttpResponse:
    """
    Get fleet comms
    :param request:
    :return:
    """

    logger.info(f"Getting formup locations for user {request.user}")

    fleet_comms = FleetComm.objects.filter(is_enabled=True).order_by("name")

    return render(
        request,
        "fleetpings/partials/form/segments/fleet-comms.html",
        {"fleet_comms": fleet_comms},
    )


@login_required
@permission_required("fleetpings.basic_access")
def ajax_get_fleet_doctrines(request: WSGIRequest) -> HttpResponse:
    """
    Get fleet doctrines for the current user
    :param request:
    :return:
    """

    logger.info(f"Getting fleet doctrines for user {request.user}")

    use_fleet_doctrines = False
    if (
        fittings_installed() is True
        and AA_FLEETPINGS_USE_DOCTRINES_FROM_FITTINGS_MODULE is True
    ):
        use_fleet_doctrines = True

    # Get doctrines
    if use_fleet_doctrines is True:
        groups = request.user.groups.all()
        doctrines = _get_docs_qs(request, groups).order_by("name")
    else:
        doctrines = (
            FleetDoctrine.objects.filter(
                Q(restricted_to_group__in=request.user.groups.all())
                | Q(restricted_to_group__isnull=True),
                is_enabled=True,
            )
            .distinct()
            .order_by("name")
        )

    return render(
        request,
        "fleetpings/partials/form/segments/fleet-doctrine.html",
        {"doctrines": doctrines, "use_fleet_doctrines": use_fleet_doctrines},
    )


@login_required
@permission_required("fleetpings.basic_access")
def _create_optimer(request: WSGIRequest, ping_context: dict):
    """
    Adding the planned fleet to the optimers
    :param request:
    :param ping_context:
    :return:
    """

    # Alliance Auth
    from allianceauth.optimer.models import OpTimer

    post_time = timezone.now()
    character = request.user.profile.main_character

    OpTimer(
        doctrine=ping_context["doctrine"]["name"],
        system=ping_context["formup_location"],
        start=ping_context["formup_time"]["datetime_string"],
        duration="N/A",
        operation_name=ping_context["fleet_name"],
        fc=ping_context["fleet_commander"],
        post_time=post_time,
        eve_character=character,
    ).save()

    logger.info(f"Optimer created by user {request.user}")


@login_required
@permission_required("fleetpings.basic_access")
def _create_aasrp_link(request: WSGIRequest, ping_context: dict) -> dict:
    """
    Create an SRP link in AA-SRP
    :param request:
    :param srp_data:
    :return:
    """

    # Third Party
    from aasrp.models import AaSrpLink

    # Django
    from django.utils.crypto import get_random_string

    post_time = timezone.now()
    creator = request.user.profile.main_character
    srp_code = get_random_string(length=16)

    AaSrpLink(
        srp_name=ping_context["fleet_name"],
        fleet_time=post_time,
        fleet_doctrine=ping_context["doctrine"]["name"],
        srp_code=srp_code,
        fleet_commander=creator,
        creator=request.user,
    ).save()

    logger.info(f"SRP Link created by user {request.user}")

    return {
        "success": True,
        "code": srp_code,
        "link": reverse_absolute("aasrp:request_srp", [srp_code]),
    }


@login_required
@permission_required("fleetpings.basic_access")
def _create_allianceauth_srp_link(request: WSGIRequest, ping_context: dict) -> dict:
    """
    Create an SRP link in Alliance Auth SRP
    :param request:
    :param ping_context:
    :return:
    """

    # Alliance Auth
    from allianceauth.srp.models import SrpFleetMain
    from allianceauth.srp.views import random_string

    post_time = timezone.now()
    creator = request.user.profile.main_character
    srp_code = random_string(8)

    SrpFleetMain(
        fleet_name=ping_context["fleet_name"],
        fleet_doctrine=ping_context["doctrine"]["name"],
        fleet_time=post_time,
        fleet_srp_code=srp_code,
        fleet_commander=creator,
    ).save()

    logger.info(f"SRP Link created by user {request.user}")

    return {
        "success": True,
        "code": srp_code,
        "link": reverse_absolute("srp:request", [srp_code]),
    }


@login_required
@permission_required("fleetpings.basic_access")
def _create_srp_link(request: WSGIRequest, ping_context: dict) -> dict:
    """
    Create an SRP link on fleetping with formup === now and SRP === yes
    :param request:
    :param ping_context:
    :return:
    """

    if ping_context["fleet_name"] and ping_context["doctrine"]["name"]:
        # Create aasrp link (prioritized app)
        if srp_module_is("aasrp") and can_add_srp_links(
            request=request, module_name="aasrp"
        ):
            aasrp_info = _create_aasrp_link(request=request, ping_context=ping_context)

            return aasrp_info

        # Create allianceauth.srp link
        if srp_module_is("allianceauth.srp") and can_add_srp_links(
            request=request, module_name="allianceauth.srp"
        ):
            allianceauth_srp_info = _create_allianceauth_srp_link(
                request=request, ping_context=ping_context
            )

            return allianceauth_srp_info

    return {
        "success": False,
        "message": _("Not all mandatory information available to create an SRP link."),
    }


@login_required
@permission_required("fleetpings.basic_access")
def ajax_create_fleet_ping(request: WSGIRequest) -> HttpResponse:
    """
    Create the fleet ping
    :param request:
    :return:
    """

    context = {}
    ping_context = {}
    success = False

    if request.method == "POST":
        form = FleetPingForm(request.POST)

        if form.is_valid():
            logger.info("Fleet ping information received")

            # Get ping context
            ping_context = get_ping_context_from_form_data(form_data=form.cleaned_data)

            # Create optimer is requested
            if optimer_installed() and ping_context["create_optimer"]:
                _create_optimer(
                    request=request,
                    ping_context=ping_context,
                )

                context["message"] = str(
                    _("Fleet operations timer has been created ...")
                )

            # Create SRP link if requested
            if srp_module_installed() and ping_context["srp"]["create_srp_link"]:
                ping_context["srp"]["link"] = _create_srp_link(
                    request=request,
                    ping_context=ping_context,
                )

                context["message"] = str(_("SRP link has been created ..."))

            # If we have a Discord webhook, ping it
            if ping_context["ping_channel"]["webhook"]:
                ping_discord_webhook(ping_context=ping_context, user=request.user)

            logger.info(f"Fleet ping created by user {request.user}")

            ping_context["request"] = request

            context["ping_context"] = render_to_string(
                "fleetpings/partials/ping/copy-paste-text.html", ping_context
            )
            success = True
        else:
            context["message"] = str(_("Form invalid. Please check your input."))
    else:
        context["message"] = str(_("No form data submitted."))

    context["success"] = success

    return HttpResponse(json.dumps(context), content_type="application/json")
